//-----------------------------------------------------------------
// Compute the hypergeometric distribution via a computer program 
// base on random numbers, and use  the corresponding urn model: 
// draw n balls from an urn N of white (K) and black (N-K) balls 
// without replacement. 
// The hypergeometric probability distribution for drawing 
// k white balls is P(X=k)=f(k;N,K,n)=(K_C_k)*(N-k_C_n-k)/(N_C_n)
// Crosscheck that your statistical computer experiment reproduces 
// this probability distribution, for N=100, fixed and various K,n.
//-----------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>

int irand(int N){
  int ran = rand()%N;
  return ran;
}

double factorial(int n){
  double f = 1.0 ;
  for(int i = 2 ; i <= n; i++){
    f*=i;
  }
  return f;
} 

double combination(int n, int k){

  double C = factorial(n)/factorial(k)/factorial(n-k);
  return C;
}


int draw(int* Z, int N, int K, int k){
  int NW = 0;
  int NB = 0;

  for(int i = N ; i > N-k ; i--){
    if(irand(i) < K) { // draw white ball
      K--;
      NW++;
    }
    else { // draw black ball
      NB++;
    }
  }
  Z[NW]++;
  return k;
}


int main(int argc, char* argv[]){

  if(argc != 3){
    printf("input K, n\n");
    exit(0);
  }

  int N_Try = 10000;
  int N = 100; // total number of balls
  int K = atoi(argv[1]); // number of white balls
  int *Z = new int[N];
  for(int i = 0 ; i < N ; i++){
    Z[i] = 0 ;
  }

  int n = atoi(argv[2]); // number of draws


  for(int i = 0 ; i < N ; i++){
    Z[i] = 0 ;
  }

  FILE *fp_anal, *fp_dat;
  char fname_anal[1024], fname_dat[1024];
  sprintf(fname_anal,"anal_K%dn%d.dat",K,n);
  sprintf(fname_dat,"K%dn%d.dat",K,n);

  fp_anal = fopen(fname_anal, "w");
  fp_dat = fopen(fname_dat, "w");

  for(int i = 0 ; i < N_Try; i++){
    draw(Z, N, K, n);
  }
  for(int i = 0 ; i < n+1 ; i++){
    fprintf(fp_dat,"%d %d\n", i, Z[i]);
  }

  for(int i = 0 ; i < n+1 ; i++){
    double C = combination(K,i)*combination(N-K, n-i)/combination(N,n);
    fprintf(fp_anal,"%d %lf\n",i, N_Try*C);
  }

  delete[] Z;

  return 0;
}
